@@ -16,7 +16,7 @@ package client
 
 import (
 	"bytes"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"testing"
 
@@ -30,13 +30,13 @@ func TestResponse(t *testing.T) {
 	under.Status = "the status message"
 	under.StatusCode = 392
 	under.Header = make(http.Header)
-	under.Header.Set("Blah", "blah blah")
-	under.Body = ioutil.NopCloser(bytes.NewBufferString("some content"))
+	under.Header.Set("Blah", "blahblah")
+	under.Body = io.NopCloser(bytes.NewBufferString("some content"))
 
 	var resp runtime.ClientResponse = response{under}
 	assert.EqualValues(t, under.StatusCode, resp.Code())
 	assert.Equal(t, under.Status, resp.Message())
-	assert.Equal(t, "blah blah", resp.GetHeader("blah"))
-	assert.Equal(t, []string{"blah blah"}, resp.GetHeaders("blah"))
+	assert.Equal(t, "blahblah", resp.GetHeader("blah"))
+	assert.Equal(t, []string{"blahblah"}, resp.GetHeaders("blah"))
 	assert.Equal(t, under.Body, resp.Body())
 }
