@@ -15,7 +15,9 @@
 package middleware
 
 import (
+	stdcontext "context"
 	"errors"
+	"fmt"
 	"net/http"
 	"net/http/httptest"
 	"strings"
@@ -24,11 +26,11 @@ import (
 	apierrors "github.com/go-openapi/errors"
 	"github.com/go-openapi/loads"
 	"github.com/go-openapi/loads/fmts"
-	"github.com/stretchr/testify/assert"
-
 	"github.com/go-openapi/runtime"
 	"github.com/go-openapi/runtime/internal/testing/petstore"
 	"github.com/go-openapi/runtime/middleware/untyped"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 type stubBindRequester struct {
@@ -45,8 +47,8 @@ func (s *stubOperationHandler) ParameterModel() interface{} {
 	return nil
 }
 
-func (s *stubOperationHandler) Handle(params interface{}) (interface{}, error) {
-	return nil, nil
+func (s *stubOperationHandler) Handle(_ interface{}) (interface{}, error) {
+	return map[string]interface{}{}, nil
 }
 
 func init() {
@@ -56,7 +58,7 @@ func init() {
 func assertAPIError(t *testing.T, wantCode int, err error) {
 	t.Helper()
 
-	assert.NotNil(t, err)
+	require.Error(t, err)
 
 	ce, ok := err.(*apierrors.CompositeError)
 	assert.True(t, ok)
@@ -69,77 +71,285 @@ func assertAPIError(t *testing.T, wantCode int, err error) {
 
 func TestContentType_Issue264(t *testing.T) {
 	swspec, err := loads.Spec("../fixtures/bugs/264/swagger.yml")
-	if assert.NoError(t, err) {
-		api := untyped.NewAPI(swspec)
-		api.RegisterConsumer("application/json", runtime.JSONConsumer())
-		api.RegisterProducer("application/json", runtime.JSONProducer())
-		api.RegisterOperation("delete", "/key/{id}", new(stubOperationHandler))
+	require.NoError(t, err)
 
-		handler := Serve(swspec, api)
-		request, _ := http.NewRequest("DELETE", "/key/1", nil)
-		recorder := httptest.NewRecorder()
-		handler.ServeHTTP(recorder, request)
-		assert.Equal(t, 200, recorder.Code)
-	}
+	api := untyped.NewAPI(swspec)
+	api.RegisterConsumer(applicationJSON, runtime.JSONConsumer())
+	api.RegisterProducer(applicationJSON, runtime.JSONProducer())
+	api.RegisterOperation("delete", "/key/{id}", new(stubOperationHandler))
+
+	handler := Serve(swspec, api)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodDelete, "/key/1", nil)
+	require.NoError(t, err)
+
+	recorder := httptest.NewRecorder()
+	handler.ServeHTTP(recorder, request)
+	assert.Equal(t, http.StatusOK, recorder.Code)
 }
 
 func TestContentType_Issue172(t *testing.T) {
 	swspec, err := loads.Spec("../fixtures/bugs/172/swagger.yml")
-	if assert.NoError(t, err) {
-		api := untyped.NewAPI(swspec)
-		api.RegisterConsumer("application/vnd.cia.v1+json", runtime.JSONConsumer())
-		api.RegisterProducer("application/vnd.cia.v1+json", runtime.JSONProducer())
-		api.RegisterOperation("get", "/pets", new(stubOperationHandler))
+	require.NoError(t, err)
 
-		handler := Serve(swspec, api)
-		request, _ := http.NewRequest("GET", "/pets", nil)
-		request.Header.Add("Accept", "application/json+special")
-		recorder := httptest.NewRecorder()
-		handler.ServeHTTP(recorder, request)
-		assert.Equal(t, http.StatusNotAcceptable, recorder.Code)
+	api := untyped.NewAPI(swspec)
+	api.RegisterConsumer("application/vnd.cia.v1+json", runtime.JSONConsumer())
+	api.RegisterProducer("application/vnd.cia.v1+json", runtime.JSONProducer())
+	api.RegisterOperation("get", "/pets", new(stubOperationHandler))
 
-		// acceptable as defined as default by the API (not explicit in the spec)
-		request.Header.Add("Accept", "application/json")
-		recorder = httptest.NewRecorder()
-		handler.ServeHTTP(recorder, request)
-		assert.Equal(t, http.StatusOK, recorder.Code)
-	}
+	handler := Serve(swspec, api)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/pets", nil)
+	require.NoError(t, err)
+
+	request.Header.Add("Accept", "application/json+special")
+	recorder := httptest.NewRecorder()
+	handler.ServeHTTP(recorder, request)
+	assert.Equal(t, http.StatusNotAcceptable, recorder.Code)
+
+	// acceptable as defined as default by the API (not explicit in the spec)
+	request.Header.Add("Accept", applicationJSON)
+	recorder = httptest.NewRecorder()
+	handler.ServeHTTP(recorder, request)
+	assert.Equal(t, http.StatusOK, recorder.Code)
 }
 
 func TestContentType_Issue174(t *testing.T) {
 	swspec, err := loads.Spec("../fixtures/bugs/174/swagger.yml")
-	if assert.NoError(t, err) {
-		api := untyped.NewAPI(swspec)
-		api.RegisterConsumer("application/json", runtime.JSONConsumer())
-		api.RegisterProducer("application/json", runtime.JSONProducer())
-		api.RegisterOperation("get", "/pets", new(stubOperationHandler))
+	require.NoError(t, err)
 
-		handler := Serve(swspec, api)
-		request, _ := http.NewRequest("GET", "/pets", nil)
-		recorder := httptest.NewRecorder()
-		handler.ServeHTTP(recorder, request)
-		assert.Equal(t, http.StatusOK, recorder.Code)
-	}
+	api := untyped.NewAPI(swspec)
+	api.RegisterConsumer(applicationJSON, runtime.JSONConsumer())
+	api.RegisterProducer(applicationJSON, runtime.JSONProducer())
+	api.RegisterOperation("get", "/pets", new(stubOperationHandler))
+
+	handler := Serve(swspec, api)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/pets", nil)
+	require.NoError(t, err)
+
+	recorder := httptest.NewRecorder()
+	handler.ServeHTTP(recorder, request)
+	assert.Equal(t, http.StatusOK, recorder.Code)
 }
 
+const (
+	testHost = "https://localhost:8080"
+
+	// how to get the spec document?
+	defaultSpecPath = "/swagger.json"
+	defaultSpecURL  = testHost + defaultSpecPath
+	// how to get the UI asset?
+	defaultUIURL = testHost + "/api/docs"
+)
+
 func TestServe(t *testing.T) {
 	spec, api := petstore.NewAPI(t)
 	handler := Serve(spec, api)
 
-	// serve spec document
-	request, _ := http.NewRequest("GET", "http://localhost:8080/swagger.json", nil)
-	request.Header.Add("Content-Type", runtime.JSONMime)
-	request.Header.Add("Accept", runtime.JSONMime)
-	recorder := httptest.NewRecorder()
+	t.Run("serve spec document", func(t *testing.T) {
+		request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultSpecURL, nil)
+		require.NoError(t, err)
 
-	handler.ServeHTTP(recorder, request)
-	assert.Equal(t, 200, recorder.Code)
+		request.Header.Add("Content-Type", runtime.JSONMime)
+		request.Header.Add("Accept", runtime.JSONMime)
+		recorder := httptest.NewRecorder()
 
-	request, _ = http.NewRequest("GET", "http://localhost:8080/swagger-ui", nil)
-	recorder = httptest.NewRecorder()
+		handler.ServeHTTP(recorder, request)
+		assert.Equal(t, http.StatusOK, recorder.Code)
+	})
 
-	handler.ServeHTTP(recorder, request)
-	assert.Equal(t, 404, recorder.Code)
+	t.Run("should not find UI there", func(t *testing.T) {
+		request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, testHost+"/swagger-ui", nil)
+		require.NoError(t, err)
+		recorder := httptest.NewRecorder()
+
+		handler.ServeHTTP(recorder, request)
+		assert.Equal(t, http.StatusNotFound, recorder.Code)
+	})
+
+	t.Run("should find UI here", func(t *testing.T) {
+		request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultUIURL, nil)
+		require.NoError(t, err)
+		recorder := httptest.NewRecorder()
+
+		handler.ServeHTTP(recorder, request)
+		assert.Equal(t, http.StatusOK, recorder.Code)
+
+		htmlResponse := recorder.Body.String()
+		assert.Containsf(t, htmlResponse, "<title>Swagger Petstore</title>", "should default to the API's title")
+		assert.Containsf(t, htmlResponse, "<redoc", "should default to Redoc UI")
+		assert.Containsf(t, htmlResponse, "spec-url='/swagger.json'>", "should default to /swagger.json spec document")
+	})
+}
+
+func TestServeWithUIs(t *testing.T) {
+	spec, api := petstore.NewAPI(t)
+	ctx := NewContext(spec, api, nil)
+
+	const (
+		alternateSpecURL  = testHost + "/specs/petstore.json"
+		alternateSpecPath = "/specs/petstore.json"
+		alternateUIURL    = testHost + "/ui/docs"
+	)
+
+	uiOpts := []UIOption{
+		WithUIBasePath("ui"), // override the base path from the spec, implies /ui
+		WithUIPath("docs"),
+		WithUISpecURL("/specs/petstore.json"),
+	}
+
+	t.Run("with APIHandler", func(t *testing.T) {
+		t.Run("with defaults", func(t *testing.T) {
+			handler := ctx.APIHandler(nil)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Containsf(t, htmlResponse, "<redoc", "should default to Redoc UI")
+			})
+
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+
+		t.Run("with options", func(t *testing.T) {
+			handler := ctx.APIHandler(nil, uiOpts...)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Contains(t, htmlResponse, fmt.Sprintf("<redoc spec-url='%s'></redoc>", alternateSpecPath))
+			})
+
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+	})
+
+	t.Run("with APIHandlerSwaggerUI", func(t *testing.T) {
+		t.Run("with defaults", func(t *testing.T) {
+			handler := ctx.APIHandlerSwaggerUI(nil)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Contains(t, htmlResponse, fmt.Sprintf(`url: '%s',`, strings.ReplaceAll(defaultSpecPath, `/`, `\/`)))
+			})
+
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+
+		t.Run("with options", func(t *testing.T) {
+			handler := ctx.APIHandlerSwaggerUI(nil, uiOpts...)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Contains(t, htmlResponse, fmt.Sprintf(`url: '%s',`, strings.ReplaceAll(alternateSpecPath, `/`, `\/`)))
+			})
+
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+	})
+
+	t.Run("with APIHandlerRapiDoc", func(t *testing.T) {
+		t.Run("with defaults", func(t *testing.T) {
+			handler := ctx.APIHandlerRapiDoc(nil)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Contains(t, htmlResponse, fmt.Sprintf("<rapi-doc spec-url=%q></rapi-doc>", defaultSpecPath))
+			})
+
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, defaultSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+
+		t.Run("with options", func(t *testing.T) {
+			handler := ctx.APIHandlerRapiDoc(nil, uiOpts...)
+
+			t.Run("should find UI", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateUIURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+
+				htmlResponse := recorder.Body.String()
+				assert.Contains(t, htmlResponse, fmt.Sprintf("<rapi-doc spec-url=%q></rapi-doc>", alternateSpecPath))
+			})
+			t.Run("should find spec", func(t *testing.T) {
+				request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, alternateSpecURL, nil)
+				require.NoError(t, err)
+				recorder := httptest.NewRecorder()
+
+				handler.ServeHTTP(recorder, request)
+				assert.Equal(t, http.StatusOK, recorder.Code)
+			})
+		})
+	})
 }
 
 func TestContextAuthorize(t *testing.T) {
@@ -147,16 +357,18 @@ func TestContextAuthorize(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := runtime.JSONRequest("GET", "/api/pets", nil)
+	request, err := runtime.JSONRequest(http.MethodGet, "/api/pets", nil)
+	require.NoError(t, err)
+	request = request.WithContext(stdcontext.Background())
 
 	ri, reqWithCtx, ok := ctx.RouteInfo(request)
 	assert.True(t, ok)
-	assert.NotNil(t, reqWithCtx)
+	require.NotNil(t, reqWithCtx)
 
 	request = reqWithCtx
 
 	p, reqWithCtx, err := ctx.Authorize(request, ri)
-	assert.Error(t, err)
+	require.Error(t, err)
 	assert.Nil(t, p)
 	assert.Nil(t, reqWithCtx)
 
@@ -165,7 +377,7 @@ func TestContextAuthorize(t *testing.T) {
 
 	request.SetBasicAuth("wrong", "wrong")
 	p, reqWithCtx, err = ctx.Authorize(request, ri)
-	assert.Error(t, err)
+	require.Error(t, err)
 	assert.Nil(t, p)
 	assert.Nil(t, reqWithCtx)
 
@@ -174,9 +386,9 @@ func TestContextAuthorize(t *testing.T) {
 
 	request.SetBasicAuth("admin", "admin")
 	p, reqWithCtx, err = ctx.Authorize(request, ri)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "admin", p)
-	assert.NotNil(t, reqWithCtx)
+	require.NotNil(t, reqWithCtx)
 
 	// Assign the new returned request to follow with the test
 	request = reqWithCtx
@@ -199,11 +411,13 @@ func TestContextAuthorize_WithAuthorizer(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := runtime.JSONRequest("POST", "/api/pets", nil)
+	request, err := runtime.JSONRequest(http.MethodPost, "/api/pets", nil)
+	require.NoError(t, err)
+	request = request.WithContext(stdcontext.Background())
 
 	ri, reqWithCtx, ok := ctx.RouteInfo(request)
 	assert.True(t, ok)
-	assert.NotNil(t, reqWithCtx)
+	require.NotNil(t, reqWithCtx)
 
 	request = reqWithCtx
 
@@ -215,13 +429,13 @@ func TestContextAuthorize_WithAuthorizer(t *testing.T) {
 
 	request.SetBasicAuth("admin", "admin")
 	p, reqWithCtx, err = ctx.Authorize(request, ri)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "admin", p)
-	assert.NotNil(t, reqWithCtx)
+	require.NotNil(t, reqWithCtx)
 
 	request.SetBasicAuth("anyother", "anyother")
 	p, reqWithCtx, err = ctx.Authorize(request, ri)
-	assert.Error(t, err)
+	require.Error(t, err)
 	ae, ok := err.(apierrors.Error)
 	assert.True(t, ok)
 	assert.Equal(t, http.StatusForbidden, int(ae.Code()))
@@ -234,7 +448,7 @@ func TestContextNegotiateContentType(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("POST", "/api/pets", nil)
+	request, _ := http.NewRequestWithContext(stdcontext.Background(), http.MethodPost, "/api/pets", nil)
 	// request.Header.Add("Accept", "*/*")
 	request.Header.Add("content-type", "text/html")
 
@@ -253,23 +467,26 @@ func TestContextBindValidRequest(t *testing.T) {
 	ctx.router = DefaultRouter(spec, ctx.api)
 
 	// invalid content-type value
-	request, _ := http.NewRequest("POST", "/api/pets", strings.NewReader(`{"name":"dog"}`))
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodPost, "/api/pets", strings.NewReader(`{"name":"dog"}`))
+	require.NoError(t, err)
 	request.Header.Add("content-type", "/json")
 
 	ri, request, _ := ctx.RouteInfo(request)
 	assertAPIError(t, 400, ctx.BindValidRequest(request, ri, new(stubBindRequester)))
 
 	// unsupported content-type value
-	request, _ = http.NewRequest("POST", "/api/pets", strings.NewReader(`{"name":"dog"}`))
+	request, err = http.NewRequestWithContext(stdcontext.Background(), http.MethodPost, "/api/pets", strings.NewReader(`{"name":"dog"}`))
+	require.NoError(t, err)
 	request.Header.Add("content-type", "text/html")
 
 	ri, request, _ = ctx.RouteInfo(request)
 	assertAPIError(t, http.StatusUnsupportedMediaType, ctx.BindValidRequest(request, ri, new(stubBindRequester)))
 
 	// unacceptable accept value
-	request, _ = http.NewRequest("POST", "/api/pets", nil)
+	request, err = http.NewRequestWithContext(stdcontext.Background(), http.MethodPost, "/api/pets", nil)
+	require.NoError(t, err)
 	request.Header.Add("Accept", "application/vnd.cia.v1+json")
-	request.Header.Add("content-type", "application/json")
+	request.Header.Add("content-type", applicationJSON)
 
 	ri, request, _ = ctx.RouteInfo(request)
 	assertAPIError(t, http.StatusNotAcceptable, ctx.BindValidRequest(request, ri, new(stubBindRequester)))
@@ -277,19 +494,19 @@ func TestContextBindValidRequest(t *testing.T) {
 
 func TestContextBindValidRequest_Issue174(t *testing.T) {
 	spec, err := loads.Spec("../fixtures/bugs/174/swagger.yml")
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	api := untyped.NewAPI(spec)
-	api.RegisterConsumer("application/json", runtime.JSONConsumer())
-	api.RegisterProducer("application/json", runtime.JSONProducer())
+	api.RegisterConsumer(applicationJSON, runtime.JSONConsumer())
+	api.RegisterProducer(applicationJSON, runtime.JSONProducer())
 	api.RegisterOperation("get", "/pets", new(stubOperationHandler))
 
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("GET", "/pets", nil)
+	request, _ := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/pets", nil)
 	ri, request, _ := ctx.RouteInfo(request)
-	assert.NoError(t, ctx.BindValidRequest(request, ri, new(stubBindRequester)))
+	require.NoError(t, ctx.BindValidRequest(request, ri, new(stubBindRequester)))
 }
 
 func TestContextBindAndValidate(t *testing.T) {
@@ -297,7 +514,7 @@ func TestContextBindAndValidate(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("POST", "/api/pets", nil)
+	request, _ := http.NewRequestWithContext(stdcontext.Background(), http.MethodPost, "/api/pets", nil)
 	request.Header.Add("Accept", "*/*")
 	request.Header.Add("content-type", "text/html")
 	request.ContentLength = 1
@@ -308,7 +525,7 @@ func TestContextBindAndValidate(t *testing.T) {
 	ri, request, _ := ctx.RouteInfo(request)
 	data, request, result := ctx.BindAndValidate(request, ri) // this requires a much more thorough test
 	assert.NotNil(t, data)
-	assert.NotNil(t, result)
+	require.Error(t, result)
 
 	v, ok := request.Context().Value(ctxBoundParams).(*validation)
 	assert.True(t, ok)
@@ -323,19 +540,19 @@ func TestContextBindAndValidate(t *testing.T) {
 func TestContextRender(t *testing.T) {
 	ct := runtime.JSONMime
 	spec, api := petstore.NewAPI(t)
-
 	assert.NotNil(t, spec)
 	assert.NotNil(t, api)
+
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("GET", "/api/pets", nil)
+	request, _ := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/api/pets", nil)
 	request.Header.Set(runtime.HeaderAccept, ct)
 	ri, request, _ := ctx.RouteInfo(request)
 
 	recorder := httptest.NewRecorder()
 	ctx.Respond(recorder, request, []string{ct}, ri, map[string]interface{}{"name": "hello"})
-	assert.Equal(t, 200, recorder.Code)
+	assert.Equal(t, http.StatusOK, recorder.Code)
 	assert.Equal(t, "{\"name\":\"hello\"}\n", recorder.Body.String())
 
 	recorder = httptest.NewRecorder()
@@ -347,17 +564,19 @@ func TestContextRender(t *testing.T) {
 
 	// Panic when route is nil and there is not a producer for the requested response format
 	recorder = httptest.NewRecorder()
-	request, _ = http.NewRequest("GET", "/api/pets", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/api/pets", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderAccept, "text/xml")
 	assert.Panics(t, func() { ctx.Respond(recorder, request, []string{}, nil, map[string]interface{}{"name": "hello"}) })
 
-	request, _ = http.NewRequest("GET", "/api/pets", nil)
+	request, err = http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/api/pets", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderAccept, ct)
 	ri, request, _ = ctx.RouteInfo(request)
 
 	recorder = httptest.NewRecorder()
 	ctx.Respond(recorder, request, []string{ct}, ri, map[string]interface{}{"name": "hello"})
-	assert.Equal(t, 200, recorder.Code)
+	assert.Equal(t, http.StatusOK, recorder.Code)
 	assert.Equal(t, "{\"name\":\"hello\"}\n", recorder.Body.String())
 
 	recorder = httptest.NewRecorder()
@@ -368,23 +587,25 @@ func TestContextRender(t *testing.T) {
 	// assert.Panics(t, func() { ctx.Respond(recorder, request, []string{ct}, ri, map[int]interface{}{1: "hello"}) })
 
 	// recorder = httptest.NewRecorder()
-	// request, _ = http.NewRequest("GET", "/pets", nil)
+	// request, _ = http.NewRequestWithContext(stdcontext.Background(),http.MethodGet, "/pets", nil)
 	// assert.Panics(t, func() { ctx.Respond(recorder, request, []string{}, ri, map[string]interface{}{"name": "hello"}) })
 
 	recorder = httptest.NewRecorder()
-	request, _ = http.NewRequest("DELETE", "/api/pets/1", nil)
+	request, err = http.NewRequestWithContext(stdcontext.Background(), http.MethodDelete, "/api/pets/1", nil)
+	require.NoError(t, err)
 	ri, request, _ = ctx.RouteInfo(request)
 	ctx.Respond(recorder, request, ri.Produces, ri, nil)
 	assert.Equal(t, 204, recorder.Code)
 }
 
 func TestContextValidResponseFormat(t *testing.T) {
-	ct := "application/json"
+	const ct = applicationJSON
 	spec, api := petstore.NewAPI(t)
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("GET", "http://localhost:8080", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "http://localhost:8080", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderAccept, ct)
 
 	// check there's nothing there
@@ -413,7 +634,8 @@ func TestContextInvalidResponseFormat(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("GET", "http://localhost:8080", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "http://localhost:8080", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderAccept, ct)
 
 	// check there's nothing there
@@ -441,7 +663,8 @@ func TestContextValidRoute(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("GET", "/api/pets", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "/api/pets", nil)
+	require.NoError(t, err)
 
 	// check there's nothing there
 	cached := request.Context().Value(ctxMatchedRoute)
@@ -470,7 +693,8 @@ func TestContextInvalidRoute(t *testing.T) {
 	ctx := NewContext(spec, api, nil)
 	ctx.router = DefaultRouter(spec, ctx.api)
 
-	request, _ := http.NewRequest("DELETE", "pets", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodDelete, "pets", nil)
+	require.NoError(t, err)
 
 	// check there's nothing there
 	cached := request.Context().Value(ctxMatchedRoute)
@@ -492,10 +716,11 @@ func TestContextInvalidRoute(t *testing.T) {
 }
 
 func TestContextValidContentType(t *testing.T) {
-	ct := "application/json"
+	ct := applicationJSON
 	ctx := NewContext(nil, nil, nil)
 
-	request, _ := http.NewRequest("GET", "http://localhost:8080", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "http://localhost:8080", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderContentType, ct)
 
 	// check there's nothing there
@@ -504,7 +729,7 @@ func TestContextValidContentType(t *testing.T) {
 
 	// trigger the parse
 	mt, _, rCtx, err := ctx.ContentType(request)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, ct, mt)
 	assert.NotNil(t, rCtx)
 	assert.NotEqual(t, request, rCtx)
@@ -517,7 +742,7 @@ func TestContextValidContentType(t *testing.T) {
 
 	// check if the cast works and fetch from cache too
 	mt, _, rCtx, err = ctx.ContentType(request)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, ct, mt)
 	assert.Equal(t, request, rCtx)
 }
@@ -526,7 +751,8 @@ func TestContextInvalidContentType(t *testing.T) {
 	ct := "application("
 	ctx := NewContext(nil, nil, nil)
 
-	request, _ := http.NewRequest("GET", "http://localhost:8080", nil)
+	request, err := http.NewRequestWithContext(stdcontext.Background(), http.MethodGet, "http://localhost:8080", nil)
+	require.NoError(t, err)
 	request.Header.Set(runtime.HeaderContentType, ct)
 
 	// check there's nothing there
@@ -535,7 +761,7 @@ func TestContextInvalidContentType(t *testing.T) {
 
 	// trigger the parse
 	mt, _, rCtx, err := ctx.ContentType(request)
-	assert.Error(t, err)
+	require.Error(t, err)
 	assert.Empty(t, mt)
 	assert.Nil(t, rCtx)
 
@@ -545,6 +771,6 @@ func TestContextInvalidContentType(t *testing.T) {
 
 	// check if the failure continues
 	_, _, rCtx, err = ctx.ContentType(request)
-	assert.Error(t, err)
+	require.Error(t, err)
 	assert.Nil(t, rCtx)
 }
